@@ -1,10 +1,10 @@
-import json, base64, os, pickle, datetime, string
+import json, base64, os, pickle, datetime, string, re
 
-__version__ = "0.1"
+__version__ = "0.1.4"
 ### from ipynb extract markdown and noted output for document organization
 
 class nbparser:
-    def __init__(self, path, output_root = "wiki"):
+    def __init__(self, path, output_root = "wiki", gitlabstyle_math = True):
         """ipynb notebook parser. from .ipynb file, generating the markdown file.
 
         Args:
@@ -14,13 +14,17 @@ class nbparser:
         self.path = path
         self.folder, self.basename  = os.path.dirname(path), os.path.splitext(os.path.basename(path))[0]
         self.notebook = json.loads("\n".join(open(path).readlines()))
-        self.outroot = output_root
-        self.outputpath = os.path.join(self.outroot, self.folder, self.basename)
-        os.makedirs(self.outputpath, exist_ok = True)
+        self.outroot = os.path.join(output_root,"source")
+        self.opath = os.path.join(self.outroot, self.folder, self.basename)
+        os.makedirs(self.opath, exist_ok = True)
         self.TOC, self.source = [],[]
         self.fig, self.code = [], []
         self.table = []
         self.ref = {}
+        self.refpath = lambda fname : os.path.join(self.basename, fname)
+        self.outputpath = lambda fname : os.path.join(self.opath, fname)
+        self.fname = self.opath+".md"
+        self.gitlab = gitlabstyle_math
     
     def conversion(self):
         self.markdown_parsing()
@@ -99,8 +103,10 @@ class nbparser:
                 if key.split("/")[0] == "image":    
                     ext = key.split("/")[1] # PNG, JPEG, etc
                     self.fig.append((data['data'][key],captions))
-                    self.source.append(f"![fig. {len(self.fig)}](fig{len(self.fig)}.{ext})  \n**Fig. {len(self.fig)}.** {captions}")
-                    self.save_fig(f"{self.outputpath}/fig{len(self.fig)}.{ext}", data['data'][key])
+                    fname = f"fig{len(self.fig)}.{ext}" # e.g.) fig1.png fig2.jpeg
+                    fignum = len(self.fig)
+                    self.source.append(f"![fig. {fignum}]({self.refpath(fname)})  \n**Fig. {fignum}.** {captions}  \n<br>\n")
+                    self.save_fig(self.outputpath(fname), data['data'][key])
 
     def save_fig(self, fname, base64fig):
         with open(fname,"wb") as f:
@@ -125,7 +131,8 @@ class nbparser:
             captions = " ".join(captions[1:])
         
         self.code.append(0) #number counting
-        self.source.append(f"**Code {len(self.code)}.** : {captions}  \n")
+        codenum = len(self.code)
+        self.source.append(f"**Code {codenum}.** : {captions}  \n")
         if output:
             self.source.append("---")
         self.source.append(f"```python\n{''.join(codecell['source'])}\n```")
@@ -140,9 +147,8 @@ class nbparser:
                         elif data.split("/")[0] == "image":
                             ext = data.split("/")[1]
                             fname = f'outputs{len(self.code)}_{i}.{ext}'
-
-                            self.source.append(f"![{ext}]({fname})")
-                            self.save_fig(f"{self.outputpath}/{fname}", out['data'][data])
+                            self.source.append(f"![{ext}]({self.refpath(fname)})")
+                            self.save_fig(f"{self.outputpath(fname)}", out['data'][data])
                             i+=1
 
                 else: #stream output
@@ -151,7 +157,8 @@ class nbparser:
                     else: # std err is ignored.
                         continue 
                 
-            self.source.append("---\n")
+            self.source.append("\n---\n")
+        self.source.append("<br>\n<br>\n")
 
             
     def parse_table(self, mdcell, tablecell):
@@ -175,8 +182,10 @@ class nbparser:
 
         self.table.append(0)
         table = [out["data"]["text/html"] for out in tablecell["outputs"] if out["output_type"]=="execute_result"][0]
+        #delete blank line
+        table = "\n".join([line for line in "".join(table).split("\n") if line]) 
         self.source.append("".join(table))
-        self.source.append(f"**Table {len(self.table)}** {captions}  \n")
+        self.source.append(f"\n**Table {len(self.table)}** {captions}  \n<br><br>\n")
 
     def toc_source(self):
         """Generate markdown source of `Table of Contents` from header in `self.TOC`.
@@ -186,14 +195,32 @@ class nbparser:
             s += ("  "*len(content[0])+"- ["+content[1]+"](#"+"-".join(content[1].lower().split())+")  \n")
         return s
     
+    def source_convert(self):
+        source = [line if not line =="!TOC" else self.toc_source() for line in self.source] # Table of Contents
+        source = "\n\n".join(source)
+        
+        # double bracket to ref
+        refexp = "\{\{[A-Za-z-]+\}\}"
+        for m in re.finditer(refexp,source):
+            if m.group(0)[2:-2] in self.ref:
+                source = source[:m.start()]+str(self.ref[m.group(0)[2:-2]])+source[m.end():]
+        if self.gitlab:
+            # math conversion
+            linemath = "\$.+\$[^\$]"
+            for m in re.finditer(linemath,source):
+                source = source[:m.start()+1] + "`" + m.group(0)[1:-2] + "`" + source[m.end()-2:]
+
+            cellmath = "\$\$.+\$\$"
+            for m in re.finditer(cellmath,source, re.DOTALL):
+                source = source[:m.start()] + "\n```math\n"+m.group(0)[2:-2]+ "\n```"+source[m.end():]
+        return source
+
     def write_markdown(self):        
+        source = self.source_convert()
+        
         ### Write markdown
-        with open(os.path.join(self.outputpath, self.basename)+".md","w") as f:
-            for line in self.source:
-                if line == "!TOC":
-                    f.write(self.toc_source())
-                else:
-                    f.write(line+"  \n\n")
+        with open(self.fname,"w") as f:
+            f.write(source)
 
 
 def toc_source(TOC):
@@ -251,23 +278,36 @@ def write_daily(note_path, mod, record):
         f.write("\n\n")
         if not today:
             f.write(f"## {datetime.date.today()}  \n")
+            f.write(f"### File : [{os.path.splitext(os.path.basename(file))[0]}]({ os.path.join('..','source',os.path.splitext(file)[0])+'.md'})  \n")
         for file in mod:
-            f.write(f"### File : [{os.path.splitext(os.path.basename(file))[0]}](../{file})  \n")
             f.write("#### Added headers:  \n")
             for header in mod[file]:
                 f.write(f"- {header}  \n")
             f.write("\n")
 
 
-shellsource = """#!/bin/bash
+shellsource = f"""#!/bin/bash
+source ~/.bashrc
+
+{"conda activate "+os.environ["CONDA_DEFAULT_ENV"] if "CONDA_DEFAULT_ENV" in os.environ else ""}
 
 git diff --name-only --cached >> __tmp__files
 
-python -m notation git-pre-commit
+python -m nbresnote git-pre-commit
 
 rm __tmp__files
+
+cd wiki
+git add *
+git commit -m "Auto commit from nbresnote `date +%F`"
 """
 
+pushsource = """#!/bin/bash
+cd wiki
+git fetch && git push
+"""
+
+
 if __name__ =="__main__":
     import sys
     #path config
@@ -289,10 +329,19 @@ if __name__ =="__main__":
                 with open(".git/hooks/pre-commit","a") as f:
                     f.write(shellsource)
                 print("Change mod of 'pre-commit' file on '.git/hooks/' into 755(rwxr-xr-x).")
-                os.chmod(".git/hooks/pre-commit", 755)
-                print("Done.")
+                os.chmod(".git/hooks/pre-commit", 0o755)
             else:
                 print("Warning : Already exists 'pre-commit' file on '.git/hooks/'.")
+            if not os.path.exists(".git/hooks/pre-push"):
+                print("Creating 'pre-push' file on '.git/hooks/'.")
+                with open(".git/hooks/pre-push","a") as f:
+                    f.write(pushsource)
+                print("Change mod of 'pre-push' file on '.git/hooks/' into 755(rwxr-xr-x).")
+                os.chmod(".git/hooks/pre-push", 0o755)
+            else:
+                print("Warning : Already exists 'pre-push' file on '.git/hooks/'.")
+            print("Done.")
+            
             exit(0)
         else:
             targets = sys.argv[1:]
